@@ -30,13 +30,13 @@ int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size) {
 
   struct mg_http_message hm;
   if (mg_http_parse((const char *) data, size, &hm) > 0) {
-    mg_crc32(0, hm.method.ptr, hm.method.len);
-    mg_crc32(0, hm.uri.ptr, hm.uri.len);
-    mg_crc32(0, hm.uri.ptr, hm.uri.len);
+    mg_crc32(0, hm.method.buf, hm.method.len);
+    mg_crc32(0, hm.uri.buf, hm.uri.len);
+    mg_crc32(0, hm.uri.buf, hm.uri.len);
     for (size_t i = 0; i < sizeof(hm.headers) / sizeof(hm.headers[0]); i++) {
       struct mg_str *k = &hm.headers[i].name, *v = &hm.headers[i].value;
-      mg_crc32(0, k->ptr, k->len);
-      mg_crc32(0, v->ptr, v->len);
+      mg_crc32(0, k->buf, k->len);
+      mg_crc32(0, v->buf, v->len);
     }
   }
   mg_http_parse(NULL, 0, &hm);
@@ -51,15 +51,15 @@ int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size) {
 
   struct mg_mqtt_message mm;
   if (mg_mqtt_parse(data, size, 0, &mm) == MQTT_OK) {
-    mg_crc32(0, mm.topic.ptr, mm.topic.len);
-    mg_crc32(0, mm.data.ptr, mm.data.len);
-    mg_crc32(0, mm.dgram.ptr, mm.dgram.len);
+    mg_crc32(0, mm.topic.buf, mm.topic.len);
+    mg_crc32(0, mm.data.buf, mm.data.len);
+    mg_crc32(0, mm.dgram.buf, mm.dgram.len);
   }
   mg_mqtt_parse(NULL, 0, 0, &mm);
   if (mg_mqtt_parse(data, size, 5, &mm) == MQTT_OK) {
-    mg_crc32(0, mm.topic.ptr, mm.topic.len);
-    mg_crc32(0, mm.data.ptr, mm.data.len);
-    mg_crc32(0, mm.dgram.ptr, mm.dgram.len);
+    mg_crc32(0, mm.topic.buf, mm.topic.len);
+    mg_crc32(0, mm.data.buf, mm.data.len);
+    mg_crc32(0, mm.dgram.buf, mm.dgram.len);
   }
   mg_mqtt_parse(NULL, 0, 5, &mm);
 
@@ -72,10 +72,10 @@ int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size) {
   mg_base64_encode(data, size, buf, sizeof(buf));
   mg_base64_encode(NULL, 0, buf, sizeof(buf));
 
-  mg_globmatch((char *) data, size, (char *) data, size);
+  mg_match(mg_str_n((char *) data, size), mg_str_n((char *) data, size), NULL);
 
-  struct mg_str k, v, s = mg_str_n((char *) data, size);
-  while (mg_span(s, &k, &v, ',')) k.len = v.len = 0;
+  struct mg_str entry, s = mg_str_n((char *) data, size);
+  while (mg_span(s, &entry, &s, ',')) entry.len = 0;
 
   int n;
   mg_json_get(mg_str_n((char *) data, size), "$", &n);
@@ -123,13 +123,12 @@ int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size) {
 int main(int argc, char *argv[]) {
   int res = EXIT_FAILURE;
   if (argc > 1) {
-    size_t len = 0;
-    char *buf = mg_file_read(&mg_fs_posix, argv[1], &len);
-    if (buf != NULL) {
-      LLVMFuzzerTestOneInput((uint8_t *) buf, len);
+    struct mg_str data = mg_file_read(&mg_fs_posix, argv[1]);
+    if (data.buf != NULL) {
+      LLVMFuzzerTestOneInput((uint8_t *) data.buf, data.len);
       res = EXIT_SUCCESS;
     }
-    free(buf);
+    free(data.buf);
   }
   return res;
 }
