@@ -1,4 +1,5 @@
 #define MG_ENABLE_TCPIP 1
+#define MG_ENABLE_TCPIP_DRIVER_INIT 0
 #define MG_ENABLE_SOCKET 0
 #define MG_USING_DHCP 1
 #define MG_ENABLE_PACKED_FS 0
@@ -76,7 +77,7 @@ static void f_http_fetch_query(struct mg_connection *c, int ev, void *ev_data) {
     struct mg_http_message *hm = (struct mg_http_message *) ev_data;
     http_responses_received++;
     if (!http_response_allocated) {
-      http_response = (char *) mg_strdup(hm->message).ptr;
+      http_response = mg_mprintf("%.*s", hm->message.len, hm->message.buf);
       http_response_allocated = 1;
     }
     if (http_responses_received > 0) {
@@ -173,9 +174,9 @@ static void mqtt_fn(struct mg_connection *c, int ev, void *ev_data) {
     mg_mqtt_pub(c, &pub_opts);
   } else if (ev == MG_EV_MQTT_MSG) {
     struct mg_mqtt_message *mm = (struct mg_mqtt_message *) ev_data;
-    if (mm->topic.len != strlen(s_topic) || strcmp(mm->topic.ptr, s_topic))
+    if (mm->topic.len != strlen(s_topic) || strcmp(mm->topic.buf, s_topic))
       ASSERT(0);
-    if (mm->data.len != 2 || strcmp(mm->data.ptr, "hi")) ASSERT(0);
+    if (mm->data.len != 2 || strcmp(mm->data.buf, "hi")) ASSERT(0);
     mg_mqtt_disconnect(c, NULL);
     *(bool *) c->fn_data = true;
   } else if (ev == MG_EV_CLOSE) {
