@@ -1,6 +1,7 @@
 #define MG_ENABLE_SOCKET 0
 #define MG_ENABLE_LINES 1
 #define MG_ENABLE_TCPIP 1
+#define MG_ENABLE_TCPIP_DRIVER_INIT 0
 #define MG_ENABLE_PACKED_FS 0
 
 #include "mongoose.c"
@@ -25,7 +26,7 @@ static void test_statechange(void) {
   memset(&iface, 0, sizeof(iface));
   iface.ip = mg_htonl(0x01020304);
   iface.state = MG_TCPIP_STATE_READY;
-  iface.tx.ptr = tx, iface.tx.len = sizeof(tx);
+  iface.tx.buf = tx, iface.tx.len = sizeof(tx);
   iface.driver = &mg_tcpip_driver_mock;
   onstatechange(&iface);
 }
